/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.department.controller;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.biz.modular.user.entity.BizUser;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.base.department.entity.Department;
import vip.xiaonuo.modular.base.department.param.*;
import vip.xiaonuo.modular.base.department.service.DepartmentService;
import vip.xiaonuo.sys.modular.user.entity.SysUser;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 部门控制器
 *
 * @author newspaper
 * @date  2023/12/13 08:44
 */
@Api(tags = "部门控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class DepartmentController {

    @Resource
    private DepartmentService departmentService;

    /**
     * 获取部门树
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取部门树")
    @GetMapping("/base/department/tree")
    public CommonResult<List<Tree<String>>> tree(DepartmentOrgIdParam departmentOrgIdParam) {
        return CommonResult.data(departmentService.tree(departmentOrgIdParam));
    }

    /**
     * 获取所有部门
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("获取下级部门")
    @GetMapping("/base/department/listChild")
    public CommonResult<Page<Department>> listChild(DepartmentChildParam departmentChildParam) {
        return CommonResult.data(departmentService.listChild(departmentChildParam));
    }



    /**
     * 获取部门分页
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("获取部门分页")
    @GetMapping("/base/department/page")
    public CommonResult<Page<Department>> page(DepartmentPageParam departmentPageParam) {
        return CommonResult.data(departmentService.page(departmentPageParam));
    }

    /**
     * 添加部门
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("添加部门")
    @CommonLog("添加部门")
    @PostMapping("/base/department/add")
    public CommonResult<String> add(@RequestParam @ApiParam(value = "部门名称") String departmentName,
                                    @RequestParam @ApiParam(value = "上级部门ID") String parentId,
                                    @RequestParam @ApiParam(value = "所在区域") String departmentRegion,
                                    @RequestParam @ApiParam(value = "详细地址") String detailLocation,
                                    @RequestParam @ApiParam(value = "组织di") String orgId,
                                    @RequestParam(required = false) @ApiParam(value = "部门电话") String departmentPhone,
                                    @RequestParam(required = false) @ApiParam(value = "部门负责人") String departmentManager,
                                    @RequestParam(required = false) @ApiParam(value = "描述") String remarks,
                                    @RequestParam(required = false) @ApiParam(value = "排序码") Integer sortCode,
                                    @RequestParam(value = "departmentImgUrl", required = false) @ApiParam(value = "部门图片") MultipartFile departmentImgUrl) {
        departmentService.add(departmentName,parentId,departmentRegion,detailLocation,departmentPhone,departmentManager,remarks,sortCode,departmentImgUrl,orgId);
        return CommonResult.ok();
    }

    /**
     * 编辑部门
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("编辑部门")
    @CommonLog("编辑部门")
    @PostMapping("/base/department/edit")
    public CommonResult<String> edit(@RequestParam @ApiParam(value = "部门ID") String id,
                                     @RequestParam @ApiParam(value = "部门名称") String departmentName,
                                     @RequestParam @ApiParam(value = "上级部门ID") String parentId,
                                     @RequestParam @ApiParam(value = "所在区域") String departmentRegion,
                                     @RequestParam @ApiParam(value = "详细地址") String detailLocation,
                                     @RequestParam(required = false) @ApiParam(value = "部门电话") String departmentPhone,
                                     @RequestParam(required = false) @ApiParam(value = "部门负责人") String departmentManager,
                                     @RequestParam(required = false) @ApiParam(value = "描述") String remarks,
                                     @RequestParam(required = false) @ApiParam(value = "排序码") Integer sortCode,
                                     @RequestParam(value = "departmentImgUrl", required = false) @ApiParam(value = "部门图片") MultipartFile departmentImgUrl) {
        departmentService.edit(id,departmentName,parentId,departmentRegion,detailLocation,departmentPhone,departmentManager,remarks,sortCode,departmentImgUrl);
        return CommonResult.ok();
    }

    /**
     * 删除部门
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("删除部门")
    @CommonLog("删除部门")
    @PostMapping("/base/department/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<DepartmentIdParam> departmentIdParamList) {
        departmentService.delete(departmentIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取部门详情
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("获取部门详情")
    @GetMapping("/base/department/detail")
    public CommonResult<Department> detail(@Valid DepartmentIdParam departmentIdParam) {
        return CommonResult.data(departmentService.detail(departmentIdParam));
    }

    /**
     * 获取人员选择器
     *
     * @author newspaper
     * @date  2023/12/13 08:44
     */
    @ApiOperationSupport(order = 7)
    @ApiOperation("获取人员选择器")
    @GetMapping("/base/department/userSelector")
    public CommonResult<List<BizUser>> userSelector(DepartmentOrgIdParam departmentOrgIdParam) {
        return CommonResult.data(departmentService.userSelector(departmentOrgIdParam));
    }



}
