/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.base.pigpen.controller;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.base.baseConfig.entity.BaseConfig;
import vip.xiaonuo.modular.base.baseConfig.service.BaseConfigService;
import vip.xiaonuo.modular.base.pigpen.entity.BasePigpen;
import vip.xiaonuo.modular.base.pigpen.param.*;
import vip.xiaonuo.modular.base.pigpen.service.BasePigpenService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * base_pigpen控制器
 *
 * @author newspaper
 * @date  2023/12/05 08:55
 */
@Api(tags = "栋舍控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class BasePigpenController {

    @Autowired
    private BaseConfigService baseConfigService;

    @Resource
    private BasePigpenService basePigpenService;

    /**
     * 获取base_pigpen分页
     *
     * @author newspaper
     * @date  2023/12/05 08:55
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取栋舍分页")
    @GetMapping("/biz/pigpen/page")
    public CommonResult<Page<BasePigpen>> page(BasePigpenPageParam basePigpenPageParam) {
        return CommonResult.data(basePigpenService.page(basePigpenPageParam));
    }

    /**
     * 获取base_pigpen树
     *
     * @author newspaper
     * @date  2023/12/05 08:55
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("获取栋舍树")
    @GetMapping("/biz/pigpen/tree")
    public CommonResult<List<Tree<String>>> tree(BasePigpenOrgIdParam basePigpenOrgIdParam) {
        return CommonResult.data(basePigpenService.tree(basePigpenOrgIdParam));
    }

    /**
     * 添加base_pigpen
     *
     * @author newspaper
     * @date  2023/12/05 08:55
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加栋舍")
    @CommonLog("添加栋舍")
    @PostMapping("/biz/pigpen/add")
    public CommonResult<String> add(@RequestBody @Valid BasePigpenAddParam basePigpenAddParam) {
        basePigpenService.add(basePigpenAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑base_pigpen
     *
     * @author newspaper
     * @date  2023/12/05 08:55
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑栋舍")
    @CommonLog("编辑栋舍")
    @PostMapping("/biz/pigpen/edit")
    public CommonResult<String> edit(@RequestBody @Valid BasePigpenEditParam basePigpenEditParam) {
        basePigpenService.edit(basePigpenEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除base_pigpen
     *
     * @author newspaper
     * @date  2023/12/05 08:55
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除栋舍")
    @CommonLog("删除栋舍")
    @PostMapping("/biz/pigpen/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<BasePigpenIdParam> basePigpenIdParamList) {
        basePigpenService.delete(basePigpenIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取base_pigpen详情
     *
     * @author newspaper
     * @date  2023/12/05 08:55
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取栋舍详情")
    @GetMapping("/biz/pigpen/detail")
    public CommonResult<BasePigpen> detail(@Valid BasePigpenIdParam basePigpenIdParam) {
        return CommonResult.data(basePigpenService.detail(basePigpenIdParam));
    }

    /**
     * 获取栋舍阶段选择器
     *
     * @author newspaper
     * @date  2023/12/22 9:13
     */
    @ApiOperationSupport(order = 6)
    @ApiOperation("获取栋舍阶段选择器")
    @GetMapping("/biz/pigpen/stageSelector")
    public CommonResult<List<BaseConfig>> typeSelector(BasePigpenOrgIdParam basePigpenOrgIdParam) {
        String orgId = basePigpenOrgIdParam.getOrgId();
        BaseConfig baseConfig = baseConfigService.getOne(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getConfigName, "栋舍阶段").eq(BaseConfig::getParentId, 0));
        List<BaseConfig> baseConfigs = baseConfigService.list(new QueryWrapper<BaseConfig>().lambda().eq(BaseConfig::getParentId, baseConfig.getId()).eq(BaseConfig::getOrgId, orgId));
        return CommonResult.data(baseConfigs);
    }
}
