/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigeliminate.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.bi.pigeliminate.entity.PigEliminate;
import vip.xiaonuo.modular.bi.pigeliminate.param.PigEliminateAddParam;
import vip.xiaonuo.modular.bi.pigeliminate.param.PigEliminateEditParam;
import vip.xiaonuo.modular.bi.pigeliminate.param.PigEliminateIdParam;
import vip.xiaonuo.modular.bi.pigeliminate.param.PigEliminatePageParam;
import vip.xiaonuo.modular.bi.pigeliminate.service.PigEliminateService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 淘汰信息控制器
 *
 * @author wang
 * @date  2023/12/08 09:14
 */
@Api(tags = "淘汰信息控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class PigEliminateController {

    @Resource
    private PigEliminateService pigEliminateService;

    /**
     * 获取淘汰信息分页
     *
     * @author wang
     * @date  2023/12/08 09:14
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取淘汰信息分页")
    @GetMapping("/bi/pigeliminate/page")
    public CommonResult<Page<PigEliminate>> page(PigEliminatePageParam pigEliminatePageParam) {
        return CommonResult.data(pigEliminateService.page(pigEliminatePageParam));
    }

    /**
     * 添加淘汰信息
     *
     * @author wang
     * @date  2023/12/08 09:14
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加淘汰信息")
    @CommonLog("添加淘汰信息")
    @PostMapping("/bi/pigeliminate/add")
    public CommonResult<String> add(@RequestBody @Valid PigEliminateAddParam pigEliminateAddParam) {
        return  pigEliminateService.add(pigEliminateAddParam);
    }

    /**
     * 编辑淘汰信息
     *
     * @author wang
     * @date  2023/12/08 09:14
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑淘汰信息")
    @CommonLog("编辑淘汰信息")
    @PostMapping("/bi/pigeliminate/edit")
    public CommonResult<String> edit(@RequestBody @Valid PigEliminateEditParam pigEliminateEditParam) {
        return pigEliminateService.edit(pigEliminateEditParam);
    }

    /**
     * 删除淘汰信息
     *
     * @author wang
     * @date  2023/12/08 09:14
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除淘汰信息")
    @CommonLog("删除淘汰信息")
    @PostMapping("/bi/pigeliminate/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<PigEliminateIdParam> pigEliminateIdParamList) {
        pigEliminateService.delete(pigEliminateIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取淘汰信息详情
     *
     * @author wang
     * @date  2023/12/08 09:14
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取淘汰信息详情")
    @GetMapping("/bi/pigeliminate/detail")
    public CommonResult<PigEliminate> detail(@Valid PigEliminateIdParam pigEliminateIdParam) {
        return CommonResult.data(pigEliminateService.detail(pigEliminateIdParam));
    }
}
