/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigfat.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.bi.pigfat.entity.PigFat;
import vip.xiaonuo.modular.bi.pigfat.mapper.PigFatMapper;
import vip.xiaonuo.modular.bi.pigfat.param.PigFatAddParam;
import vip.xiaonuo.modular.bi.pigfat.param.PigFatEditParam;
import vip.xiaonuo.modular.bi.pigfat.param.PigFatIdParam;
import vip.xiaonuo.modular.bi.pigfat.param.PigFatPageParam;
import vip.xiaonuo.modular.bi.pigfat.service.PigFatService;
import vip.xiaonuo.modular.bi.pigfiles.entity.PigFiles;

import java.util.List;

/**
 * 肥猪信息Service接口实现类
 *
 * @author wang
 * @date  2023/12/18 09:20
 **/
@Service
public class PigFatServiceImpl extends ServiceImpl<PigFatMapper, PigFat> implements PigFatService {

    @Autowired
    private PigFatMapper fatMapper;
    @Override
    public Page<PigFat> page(PigFatPageParam pigFatPageParam) {
        QueryWrapper<PigFat> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(pigFatPageParam.getEartag())) {
            queryWrapper.lambda().like(PigFat::getEartag, pigFatPageParam.getEartag());
        }
        if(ObjectUtil.isAllNotEmpty(pigFatPageParam.getSortField(), pigFatPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigFatPageParam.getSortOrder());
            queryWrapper.orderBy(true, pigFatPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigFatPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigFat::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult add(PigFatAddParam pigFatAddParam) {
        QueryWrapper<PigFat> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", pigFatAddParam.getOrgId()).eq("eartag", pigFatAddParam.getEartag());
        if (ObjectUtil.isEmpty(fatMapper.selectOne(queryWrapper))) {
            PigFat pigFat = BeanUtil.toBean(pigFatAddParam, PigFat.class);
            this.save(pigFat);
            return CommonResult.ok();
        } else {
            return new CommonResult(500, "耳标号已存在，请重新添加！", false);
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult edit(PigFatEditParam pigFatEditParam) {
        PigFat pigFat = this.queryEntity(pigFatEditParam.getId());
        if (ObjectUtil.isNotEmpty(pigFatEditParam.getEartag())) {
            QueryWrapper<PigFat> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id", pigFat.getOrgId())
                    .eq(StringUtils.isNotBlank(pigFatEditParam.getEartag()),"eartag",pigFatEditParam.getEartag())
                    .ne("id", pigFatEditParam.getId());
            if (ObjectUtil.isEmpty(fatMapper.selectOne(queryWrapper))) {
                BeanUtil.copyProperties(pigFatEditParam, pigFat);
                this.updateById(pigFat);
                return CommonResult.ok();
            } else {
                return new CommonResult(500, "耳标号已存在！", false);
            }
        } else {
            BeanUtil.copyProperties(pigFatEditParam, pigFat);
            this.updateById(pigFat);
            return CommonResult.ok();
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigFatIdParam> pigFatIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(pigFatIdParamList, PigFatIdParam::getId));
    }

    @Override
    public PigFat detail(PigFatIdParam pigFatIdParam) {
        return this.queryEntity(pigFatIdParam.getId());
    }

    @Override
    public PigFat queryEntity(Integer id) {
        PigFat pigFat = this.getById(id);
        if(ObjectUtil.isEmpty(pigFat)) {
            throw new CommonException("肥猪信息不存在，id值为：{}", id);
        }
        return pigFat;
    }
}
