/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigfiles.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.bi.pigfiles.entity.PigFiles;
import vip.xiaonuo.modular.bi.pigfiles.mapper.PigFilesMapper;
import vip.xiaonuo.modular.bi.pigfiles.param.PigFilesAddParam;
import vip.xiaonuo.modular.bi.pigfiles.param.PigFilesEditParam;
import vip.xiaonuo.modular.bi.pigfiles.param.PigFilesIdParam;
import vip.xiaonuo.modular.bi.pigfiles.param.PigFilesPageParam;
import vip.xiaonuo.modular.bi.pigfiles.service.PigFilesService;

import java.util.List;

/**
 * 猪只档案Service接口实现类
 *
 * @author wang
 * @date  2023/12/06 14:33
 **/
@Service
public class PigFilesServiceImpl extends ServiceImpl<PigFilesMapper, PigFiles> implements PigFilesService {

    @Autowired
    private PigFilesMapper filesMapper;
    @Override
    public Page<PigFiles> page(PigFilesPageParam pigFilesPageParam) {
        QueryWrapper<PigFiles> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(pigFilesPageParam.getEartag())) {
            queryWrapper.lambda().like(PigFiles::getEartag, pigFilesPageParam.getEartag());
        }
        if(ObjectUtil.isAllNotEmpty(pigFilesPageParam.getSortField(), pigFilesPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigFilesPageParam.getSortOrder());
            queryWrapper.orderBy(true, pigFilesPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigFilesPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigFiles::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult add(PigFilesAddParam pigFilesAddParam) {
        QueryWrapper<PigFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", pigFilesAddParam.getOrgId()).eq("eartag", pigFilesAddParam.getEartag());
        if (ObjectUtil.isEmpty(filesMapper.selectOne(queryWrapper))) {
            PigFiles pigFiles = BeanUtil.toBean(pigFilesAddParam, PigFiles.class);
            this.save(pigFiles);
            return CommonResult.ok();
        } else {
            return new CommonResult(500, "耳标号已存在，请重新添加！", false);
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult edit(PigFilesEditParam pigFilesEditParam) {
        PigFiles pigFiles = this.queryEntity(pigFilesEditParam.getId());
        if (ObjectUtil.isNotEmpty(pigFilesEditParam.getEartag())) {
            QueryWrapper<PigFiles> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id", pigFiles.getOrgId())
                    .eq(StringUtils.isNotBlank(pigFilesEditParam.getEartag()),"eartag",pigFilesEditParam.getEartag())
                    .ne("id", pigFilesEditParam.getId());
            if (ObjectUtil.isEmpty(filesMapper.selectOne(queryWrapper))) {
                BeanUtil.copyProperties(pigFilesEditParam, pigFiles);
                this.updateById(pigFiles);
                return CommonResult.ok();
            } else {
                return new CommonResult(500, "耳标号已存在！", false);
            }
        } else {
            BeanUtil.copyProperties(pigFilesEditParam, pigFiles);
            this.updateById(pigFiles);
            return CommonResult.ok();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigFilesIdParam> pigFilesIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(pigFilesIdParamList, PigFilesIdParam::getId));
    }

    @Override
    public PigFiles detail(PigFilesIdParam pigFilesIdParam) {
        return this.queryEntity(pigFilesIdParam.getId());
    }

    @Override
    public PigFiles queryEntity(Integer id) {
        PigFiles pigFiles = this.getById(id);
        if(ObjectUtil.isEmpty(pigFiles)) {
            throw new CommonException("猪只档案不存在，id值为：{}", id);
        }
        return pigFiles;
    }
}
