/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigimmunity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.bi.pigimmunity.entity.PigImmunity;
import vip.xiaonuo.modular.bi.pigimmunity.mapper.PigImmunityMapper;
import vip.xiaonuo.modular.bi.pigimmunity.param.PigImmunityAddParam;
import vip.xiaonuo.modular.bi.pigimmunity.param.PigImmunityEditParam;
import vip.xiaonuo.modular.bi.pigimmunity.param.PigImmunityIdParam;
import vip.xiaonuo.modular.bi.pigimmunity.param.PigImmunityPageParam;
import vip.xiaonuo.modular.bi.pigimmunity.service.PigImmunityService;

import java.util.Date;
import java.util.List;

/**
 * 免疫信息Service接口实现类
 *
 * @author wang
 * @date  2023/12/08 09:29
 **/
@Service
public class PigImmunityServiceImpl extends ServiceImpl<PigImmunityMapper, PigImmunity> implements PigImmunityService {

    @Override
    public Page<PigImmunity> page(PigImmunityPageParam pigImmunityPageParam) {
        QueryWrapper<PigImmunity> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(pigImmunityPageParam.getEartag())) {
            queryWrapper.lambda().like(PigImmunity::getEartag, pigImmunityPageParam.getEartag());
        }
        if(ObjectUtil.isNotEmpty(pigImmunityPageParam.getStartVaccinateDate()) && ObjectUtil.isNotEmpty(pigImmunityPageParam.getEndVaccinateDate())) {
            queryWrapper.lambda().between(PigImmunity::getVaccinateDate, pigImmunityPageParam.getStartVaccinateDate(), pigImmunityPageParam.getEndVaccinateDate());
        }
        if(ObjectUtil.isAllNotEmpty(pigImmunityPageParam.getSortField(), pigImmunityPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigImmunityPageParam.getSortOrder());
            queryWrapper.orderBy(true, pigImmunityPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigImmunityPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigImmunity::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PigImmunityAddParam pigImmunityAddParam) {
        PigImmunity pigImmunity = BeanUtil.toBean(pigImmunityAddParam, PigImmunity.class);
        pigImmunity.setCreateDate(new Date());
        this.save(pigImmunity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(PigImmunityEditParam pigImmunityEditParam) {
        PigImmunity pigImmunity = this.queryEntity(pigImmunityEditParam.getId());
        BeanUtil.copyProperties(pigImmunityEditParam, pigImmunity);
        pigImmunity.setUpdateDate(new Date());
        this.updateById(pigImmunity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigImmunityIdParam> pigImmunityIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(pigImmunityIdParamList, PigImmunityIdParam::getId));
    }

    @Override
    public PigImmunity detail(PigImmunityIdParam pigImmunityIdParam) {
        return this.queryEntity(pigImmunityIdParam.getId());
    }

    @Override
    public PigImmunity queryEntity(Integer id) {
        PigImmunity pigImmunity = this.getById(id);
        if(ObjectUtil.isEmpty(pigImmunity)) {
            throw new CommonException("免疫信息不存在，id值为：{}", id);
        }
        return pigImmunity;
    }
}
