/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.feed.feedingenv.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.feed.feedingenv.entity.FeedingEnv;
import vip.xiaonuo.modular.feed.feedingenv.mapper.FeedingEnvMapper;
import vip.xiaonuo.modular.feed.feedingenv.param.FeedingEnvAddParam;
import vip.xiaonuo.modular.feed.feedingenv.param.FeedingEnvEditParam;
import vip.xiaonuo.modular.feed.feedingenv.param.FeedingEnvIdParam;
import vip.xiaonuo.modular.feed.feedingenv.param.FeedingEnvPageParam;
import vip.xiaonuo.modular.feed.feedingenv.service.FeedingEnvService;

import java.text.DecimalFormat;
import java.util.List;

/**
 * 精准饲喂设备Service接口实现类
 *
 * @author wang
 * @date 2023/12/28 11:26
 **/
@Service
public class FeedingEnvServiceImpl extends ServiceImpl<FeedingEnvMapper, FeedingEnv> implements FeedingEnvService {

    @Autowired
    private FeedingEnvMapper envMapper;

    @Override
    public List<FeedingEnv> page(FeedingEnvPageParam feedingEnvPageParam) {
        QueryWrapper<FeedingEnv> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", feedingEnvPageParam.getOrgId());
        List<FeedingEnv> feedingEnvs = envMapper.selectList(queryWrapper);
        return feedingEnvs;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult add(FeedingEnvAddParam feedingEnvAddParam) {
        FeedingEnv feedingEnv = BeanUtil.toBean(feedingEnvAddParam, FeedingEnv.class);
        QueryWrapper<FeedingEnv> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", feedingEnv.getOrgId())
                .eq("device_code", feedingEnv.getDeviceCode())
                .or().eq("unit_id", feedingEnv.getUnitId());
        if (ObjectUtil.isNotEmpty(envMapper.selectOne(queryWrapper))) {
            return new CommonResult(500, "编号重复或者栋舍已被绑定", false);
        } else {
            this.save(feedingEnv);
            return CommonResult.ok();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CommonResult edit(FeedingEnvEditParam feedingEnvEditParam) {
        if (this.count(new QueryWrapper<FeedingEnv>().lambda().eq(FeedingEnv::getOrgId, feedingEnvEditParam.getOrgId())
                .eq(FeedingEnv::getUnitId, feedingEnvEditParam.getUnitId())) > 0) {
            return new CommonResult(500, "栋舍已被绑定", false);
        }
        if (this.count(new QueryWrapper<FeedingEnv>().lambda().eq(FeedingEnv::getOrgId, feedingEnvEditParam.getOrgId())
                .eq(FeedingEnv::getDeviceCode, feedingEnvEditParam.getDeviceCode())) > 0) {
            return new CommonResult(500, "设备编号已存在！", false);
        }
        FeedingEnv feedingEnv = this.queryEntity(feedingEnvEditParam.getId().toString());
        BeanUtil.copyProperties(feedingEnvEditParam, feedingEnv);
        this.updateById(feedingEnv);
        return CommonResult.ok();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<FeedingEnvIdParam> feedingEnvIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(feedingEnvIdParamList, FeedingEnvIdParam::getId));
    }

    @Override
    public FeedingEnv detail(FeedingEnvIdParam feedingEnvIdParam) {
        return this.queryEntity(feedingEnvIdParam.getId().toString());
    }

    @Override
    public FeedingEnv queryEntity(String id) {
        FeedingEnv feedingEnv = this.getById(id);
        if (ObjectUtil.isEmpty(feedingEnv)) {
            throw new CommonException("精准饲喂设备不存在，id值为：{}", id);
        }
        return feedingEnv;
    }

    @Override
    public JSONObject listEnv(String orgId) {
        QueryWrapper<FeedingEnv> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", orgId);
        List<FeedingEnv> envs = envMapper.selectList(queryWrapper);
        Integer onliveCount = 0;
        Integer offliveCount = 0;
        for (FeedingEnv env : envs) {
            if (env.getDeviceStatus() == 0) {
                //在线
                onliveCount = onliveCount + 1;
            }
            if (env.getDeviceStatus() == 1) {
                //离线
                offliveCount = offliveCount + 1;
            }
        }
        double v = Double.parseDouble(onliveCount.toString());
        double rank = v / Double.parseDouble(String.valueOf(envs.size()));
        DecimalFormat dft = new DecimalFormat("0.00");
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("envCount", envs.size());
        jsonObject.put("onliveCount", onliveCount);
        jsonObject.put("offliveCount", offliveCount);
        jsonObject.put("rank", dft.format(rank));
        return jsonObject;
    }
}
